"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.algo_check_expect_empty = exports.algo_check_expect_block_out_of_range = exports.algo_ensure_data = exports.checksum = exports.txIdToHexNo0x = exports.txIdToHex = exports.hexToAddress = exports.addressToHex = exports.base64ToText = exports.hexToBase64 = exports.base64ToHex = exports.hexToBase32 = exports.base32ToHex = exports.bytesToHex = exports.hexToBytes = exports.filterHashes = exports.INVALIDPUBKEYPAIRERROR = exports.INVALIDADDRESERROR = void 0;
const utils_1 = require("./utils");
const base32 = require("base32.js");
const { default: algosdk } = require("algosdk");
const crypto = require("crypto");
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////// Errors //////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const INVALIDADDRESERROR = (address) => {
    return new Error(`Invalid address: ${address}`);
};
exports.INVALIDADDRESERROR = INVALIDADDRESERROR;
const INVALIDPUBKEYPAIRERROR = (algoKeyPair) => {
    return new Error(`Invalid keypair: ${algoKeyPair.publicKey} -  ${algoKeyPair.checksum}`);
};
exports.INVALIDPUBKEYPAIRERROR = INVALIDPUBKEYPAIRERROR;
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////// Code ////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function filterHashes(trans) {
    if (trans.id) {
        return trans.id;
    }
    else {
        return "";
    }
}
exports.filterHashes = filterHashes;
///////////////////
// Bytes <-> Hex //
///////////////////
// Convert a hex string to a byte array
function hexToBytes(hex) {
    let bytes = [];
    for (let c = 0; c < hex.length; c += 2)
        bytes.push(parseInt(hex.substr(c, 2), 16));
    return new Uint8Array(bytes);
}
exports.hexToBytes = hexToBytes;
// Convert a byte array to a hex string
function bytesToHex(bytes) {
    for (var hex = [], i = 0; i < bytes.length; i++) {
        var current = bytes[i] < 0 ? bytes[i] + 256 : bytes[i];
        hex.push((current >>> 4).toString(16));
        hex.push((current & 0xf).toString(16));
    }
    return hex.join("");
}
exports.bytesToHex = bytesToHex;
////////////////////
// Base32 <-> Hex //
////////////////////
/**
 * Algos base32 to hex string
 * @param data
 * @returns
 */
function base32ToHex(data) {
    // old base 32 decoding
    const decoder = new base32.Decoder({ type: "rfc4648" });
    let out = decoder.write(data).finalize();
    return out.toString("hex");
    // hibase 32
    // return Buffer.from(hibase32.decode.asBytes(data)).toString("hex")
}
exports.base32ToHex = base32ToHex;
function hexToBase32(hex) {
    // old base 32 encoding
    const encoder = new base32.Encoder({ type: "rfc4648" });
    let bufHex = Buffer.from(hex);
    let base32Out = encoder.write(bufHex).finalize();
    return base32Out;
    // hibase 32
    // return hibase32.encode(hex)
}
exports.hexToBase32 = hexToBase32;
////////////////////
// Base64 <-> Hex //
////////////////////
/**
 * For note decoding
 * @param rawdata
 * @returns
 * @dev todo stay away from deprecated atob
 */
function base64ToHex(rawdata) {
    // let data = atob(rawdata);
    let data = Buffer.from(rawdata, "base64").toString("binary");
    let hexData = "";
    for (let i = 0; i < data.length; i++) {
        const hex = data.charCodeAt(i).toString(16);
        hexData += hex.length === 2 ? hex : "0" + hex;
    }
    return hexData;
}
exports.base64ToHex = base64ToHex;
function hexToBase64(hex) {
    if (typeof hex === "string") {
        return Buffer.from(hex, "hex").toString("base64");
    }
    else {
        return Buffer.from(bytesToHex(hex), "hex").toString("base64");
    }
}
exports.hexToBase64 = hexToBase64;
/////////////////////
// Base64 <-> Text //
/////////////////////
/**
 *
 * @dev used for transaction note parsing
 * @param rawdata
 * @returns
 */
function base64ToText(rawdata) {
    // return atob(rawdata);
    let buff = Buffer.from(rawdata, "base64");
    return buff.toString("ascii");
}
exports.base64ToText = base64ToText;
//////////////////////////
// Algo address <-> Hex //
//////////////////////////
function addressToHex(address) {
    if (!algosdk.isValidAddress(address)) {
        throw (0, exports.INVALIDADDRESERROR)(address);
    }
    let dad = algosdk.decodeAddress(address);
    const publicKey = Buffer.from(dad.publicKey).toString("hex");
    const checksum = Buffer.from(dad.checksum).toString("hex");
    return {
        publicKey: (0, utils_1.prefix0x)(publicKey),
        checksum: (0, utils_1.prefix0x)(checksum),
    };
}
exports.addressToHex = addressToHex;
function hexToAddress(algoKeyPair) {
    let conc = (0, utils_1.unPrefix0x)(algoKeyPair.publicKey) + (0, utils_1.unPrefix0x)(algoKeyPair.checksum);
    let concBytes = hexToBytes(conc);
    const address = hexToBase32(concBytes);
    if (!algosdk.isValidAddress(address)) {
        throw (0, exports.INVALIDPUBKEYPAIRERROR)(algoKeyPair);
    }
    return address;
}
exports.hexToAddress = hexToAddress;
/////////////////////////////////
// Algo transaction id <-> Hex //
/////////////////////////////////
function txIdToHex(txid) {
    return (0, utils_1.prefix0x)(base32ToHex(txid));
}
exports.txIdToHex = txIdToHex;
function txIdToHexNo0x(txid) {
    return base32ToHex(txid);
}
exports.txIdToHexNo0x = txIdToHexNo0x;
/////////////////////////////////
// Algo Checksum ////////////////
/////////////////////////////////
function checksum(str, algorithm = "sha512", encoding = "hex") {
    return crypto.createHash(algorithm).update(str, "utf8").digest(encoding);
}
exports.checksum = checksum;
////////////////////////////
//// MCC Error handling ////
////////////////////////////
function algo_ensure_data(data) {
    const error_codes = [400, 401, 404, 500, 503];
    if (error_codes.includes(data.status)) {
        throw (0, utils_1.MccError)(data);
    }
}
exports.algo_ensure_data = algo_ensure_data;
function algo_check_expect_block_out_of_range(data) {
    if (data.status === 404) {
        return true;
    }
    else {
        return false;
    }
}
exports.algo_check_expect_block_out_of_range = algo_check_expect_block_out_of_range;
function algo_check_expect_empty(data) {
    if (data.status === 404) {
        return true;
    }
    else {
        return false;
    }
}
exports.algo_check_expect_empty = algo_check_expect_empty;
//# sourceMappingURL=algoUtils.js.map